/*-----------------------------------------------------------------------------

===========================================================================
Rivista  : Cubase Magazine
Rubrica  : CUBASIS VST FACILE (15)
Articolo : 'Realizziamo un semplice Plug-In per Cubasis VST 3.0 (parte IV)'
       di: Giuseppe Varriale (admin@websoftconsulting.com)
Contenuto: analisi, personalizzazione e costruzione a scopo didattico
           di un semplice plug-in (CMGain) per Cubasis VST 3.

File     : AGain.hpp
===========================================================================

Esempio originale del VST SDK: AGain
 1999, Steinberg Soft und Hardware GmbH, All Rights Reserved


-----------------------------------------------------------------------------*/

#ifndef __AGAIN_H
#define __AGAIN_H

#include "audioeffectx.h"

//-----------------------------------------------------------------------------*/
// La classe del plug-in  derivata da quella base AudioEffectX
// in questo modo ne 'eredita' tutte le caratteristiche.

class AGain : public AudioEffectX
{
public:
	AGain(audioMasterCallback audioMaster);
	~AGain();

    // il cuore del plug-in: le funzioni accumula/rimpiazza!
	virtual void process(float **inputs, float **outputs, long sampleFrames);
	virtual void processReplacing(float **inputs, float **outputs, long sampleFrames);
    // le funzioni membro per impostare/leggere nomi di programma
	virtual void setProgramName(char *name);
	virtual void getProgramName(char *name);
    // le funz. membro per impostare/leggere un parametro
	virtual void setParameter(long index, float value);
	virtual float getParameter(long index);
    // funz. membro per ottenere letichetta del parametro
	virtual void getParameterLabel(long index, char *label);
    // funz. membro per ottenere il valore visualizzato del parametro
	virtual void getParameterDisplay(long index, char *text);
    // funz. membro per ottenere il nome del parametro
	virtual void getParameterName(long index, char *text);

protected:
    // dato membro contenente il Guadagno dellamplificatore plug-in
	float fGain;
    // dato membro contenente il nome del programma
	char programName[32];
};

//-----------------------------------------------------------------------------*/
// NB: breve glossario di riferimento

// parametro = impostazione individuale di parametri effettuabile dallutente
// programma = insieme di parametri descriventi lo stato corrente del plug-in
// banco = collezione di programmi

#endif
